#pragma once

#include "session_event.hpp"

#include <window.hpp>


/// renderable content. accepts render target as parameter, which can be a local window or a remote window
template <typename render_target>
class display_content {
private:

	std::size_t _id;
	render_target _render_device;

public:

	/// constructor. construct with content id and event listener.
	display_content(const std::size_t& id, render_target&& device)
		: _id(id)
		, _render_device(std::move(device)) {
	}

	/// retrieve the content id.
	virtual const std::size_t id() const { return _id; }

	/// retrieve the render target for this content.
	render_target& target() { return _render_device; }
	const render_target& target() const { return _render_device; }

	/// generate a local preview image of the content. by default, returns a blank bitmap but derived classes can override to provide a preview image.
	virtual bitmap make_preview_image(const window& win, const SIZE& constraints) {
		return win.compatible_bitmap({ constraints.cx, constraints.cy });
	}

	virtual void on_event(session_event evt, const unsigned int& id) {}
	virtual void render(HWND hwndMsg, UINT message_code, UINT32 flags) {}
};